/*
 * This file is part of be_cbo.
 * Copyright 2009 Benjamin Eikel
 *
 * be_cbo is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation in version 2 of the License.
 *
 * be_cbo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * The file "be_cbo/gpl-2.0.txt" contains a copy of the 
 * GNU General Public License.
 */

#ifndef APPLICATION_H
#define APPLICATION_H

#include "Shape.h"
#include <btBulletDynamicsCommon.h>
#include <osg/Texture2D>
#include <osgViewer/Viewer>

/**
 * Functions for setup, clean up and execution of the application.
 *
 * @brief Main application
 * @author Benjamin Eikel
 * @date 2009-04-15
 */
class Application {
	public:
		static const osg::ref_ptr<osg::Texture2D> g_woodTexture;
		static const osg::ref_ptr<osg::Texture2D> g_graniteTexture;

		static const float g_woodDensity; // kg / (m ^ 3)
		static const float g_graniteDensity; // kg / (m ^ 3)

		Application();
		~Application();
		void mainLoop();

		void addPhysicsNode(	osg::Texture * texture,
								Shape * shape,
								btVector3 center,
								btScalar mass,
								btVector3 velocity = btVector3(0.0f, 0.0f, 0.0f));
		void addGroundPlane();
		void addRandomBoxes();

		void togglePhysicsUpdate();

	private:
		btCollisionConfiguration * m_collisionConfiguration;
		btDispatcher * m_dispatcher;
		btBroadphaseInterface * m_broadphase;
		btConstraintSolver * m_solver;
		btDiscreteDynamicsWorld * m_world;

		void initBullet();
		void destroyBullet();

		osgViewer::Viewer m_viewer;

		void initOpenSceneGraph();
		void destroyOpenSceneGraph();

		bool m_updatePhysics;
};

#endif // APPLICATION_H
