/*
 * This file is part of be_cbo.
 * Copyright 2009 Benjamin Eikel
 *
 * be_cbo is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation in version 2 of the License.
 *
 * be_cbo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * The file "be_cbo/gpl-2.0.txt" contains a copy of the 
 * GNU General Public License.
 */

#ifndef INPUTHANDLER_H
#define INPUTHANDLER_H

#include <osg/Group>
#include <osgGA/GUIActionAdapter>
#include <osgGA/GUIEventAdapter>
#include <osgGA/GUIEventHandler>
#include <osgViewer/Viewer>

class Application;

/**
 * Handle user input. Key strokes are used to create new objects ('1', '2' and 
 * '3'), reset the scene ('r'), add new boxes ('n'), toggle physics 
 * caluclations ('p') and delete selected objects ('[backspace]' or 
 * '[delete]'). Mouse click selects and deselects an object in the 3D scene.
 *
 * @brief Handle keyboard and mouse input
 * @author Benjamin Eikel
 * @date 2009-04-15
 * @see OpenSceneGraph example osgkeyboardmouse
 */
class InputHandler : public osgGA::GUIEventHandler {
	public:
		InputHandler(Application & app);	
		bool handle(const osgGA::GUIEventAdapter & ea, osgGA::GUIActionAdapter & aa);

	private:
		Application & m_application;
		float m_mouseX;
		float m_mouseY;

		void pick(const osgGA::GUIEventAdapter & ea, osgViewer::Viewer * viewer);	
		void toggleScribe(osg::Group * parent, osg::Node * node);
};

#endif // INPUTHANDLER_H
